%% Read and preprocess data from database and save results for processing in files:
%  ResultA and ResultB
% A is the location to measure
% relative to B as reference (e.g. WebSDR Maasbree)
% PA0SIM July 2021

clear all; %#ok<CLALL>
% close all;

%% settings ====================================================================
% matchdistance: frequency has to be within xHz to match (double calls)
matchdistance=25; % Hz

%% select the data =============================================================
[reporterA,reporterB,startDT,endDT,band,limit]=selection(2);

%% collect all data from database ==============================================
[datesA,timesA,SNRA,callA,bandA,locA,freqA,azimAdb,distA]=WSPR_database(reporterA,band,startDT,endDT,limit);
[datesB,timesB,SNRB,callB,bandB,locB,freqB,azimBdb,distB]=WSPR_database(reporterB,band,startDT,endDT,limit);

% width of the data fields
wids=[size(datesA,2) size(timesA,2) size(SNRA,2) size(callA,2) ...
        size(bandA,2) size(locA,2) size(freqA,2) size(azimAdb,2) size(distA,2)];

fprintf("allData valid info found.\n");

%% catenate for compare and search =============================================
matchA=cat(2,datesA,timesA,callA,bandA);
matchB=cat(2,datesB,timesB,callB,bandB);

%% find matching (valid) signals and remove double calls in 2 min periods ======
[matchIndexA, matchIndexB] = findMatch(matchA,matchB,freqA,freqB,matchdistance,wids);

fprintf("Matching signals found.\n");

%% correct for RX frequency offset and filter for overlapping band =============
[freqAnum] = -100+convToWSPRfreq(freqA(matchIndexA,:),100,1);
[freqBnum] = -100+convToWSPRfreq(freqB(matchIndexB,:),100,1);
% Roffset is offset frequency between both receivers <<<<<
Roffset=mean(freqAnum,1)-mean(freqBnum,1);
% matching frequencies overlap
[freqAnum] = -100+convToWSPRfreq(freqA,100,1);
[freqBnum] = -100+convToWSPRfreq(freqB,100,1);
freqAnum=freqAnum-Roffset;
freqBnum=freqBnum-0;
minFreq=-0.1+min([min(freqAnum) min(freqBnum)]); % -0.1 for rounding error
maxFreq=min([max(freqAnum) max(freqBnum)]);
validAfreqIndex=find((freqAnum>=minFreq).*(freqAnum<=maxFreq));
validBfreqIndex=find((freqBnum>=minFreq).*(freqBnum<=maxFreq));

%% use only the signals with frequency in the overlapping bandwidth ============
datesA=datesA(validAfreqIndex,:);
timesA=timesA(validAfreqIndex,:);
SNRA=SNRA(validAfreqIndex,:);
callA=callA(validAfreqIndex,:);
bandA=bandA(validAfreqIndex,:);
locA=locA(validAfreqIndex,:);
freqA=freqA(validAfreqIndex,:);
azimAdb=azimAdb(validAfreqIndex,:);
distA=distA(validAfreqIndex,:);

datesB=datesB(validBfreqIndex,:);
timesB=timesB(validBfreqIndex,:);
SNRB=SNRB(validBfreqIndex,:);
callB=callB(validBfreqIndex,:);
bandB=bandB(validBfreqIndex,:);
locB=locB(validBfreqIndex,:);
freqB=freqB(validBfreqIndex,:);
azimBdb=azimBdb(validBfreqIndex,:);
distB=distB(validBfreqIndex,:);

AlldataA=cat(2,datesA,timesA,SNRA,callA,bandA,locA,freqA,azimAdb,distA);
AlldataB=cat(2,datesB,timesB,SNRB,callB,bandB,locB,freqB,azimBdb,distB);

%% catenate with signals within overlapping band for compare and search ========
matchA=cat(2,datesA,timesA,callA,bandA);
matchB=cat(2,datesB,timesB,callB,bandB);

%% find matching (valid) signals and find double calls in each 2 min period ====
[matchIndexA, matchIndexB] = findMatch(matchA,matchB,freqA,freqB,matchdistance,wids);

fprintf("Matching signals found in overlapping band.\n");

%% find (intended or unintended) multi transmitting signals by frequency deviation
[freqAnum] = -100+convToWSPRfreq(freqA(matchIndexA,:),100,1);
[freqBnum] = -100+convToWSPRfreq(freqB(matchIndexB,:),100,1);
invalidCallIndex=find(abs(freqAnum-freqBnum)>matchdistance);
matchIndexA(:,invalidCallIndex)='';
matchIndexB(:,invalidCallIndex)='';

Valid_dataA_start=AlldataA(matchIndexA,:);
Valid_dataB_start=AlldataB(matchIndexB,:);

%% save to files for processing ================================================
save('ResultsA','Valid_dataA_start', 'Valid_dataB_start','wids', ...
                        'AlldataA','matchIndexA','Roffset','minFreq');
save('ResultsB','Valid_dataA_start', 'Valid_dataB_start','wids', ...
                        'AlldataB','matchIndexB','Roffset','minFreq');
