%% Process the preprocessed WSPR data and plot the results
% run WSPR_preprocessing script first!
% PA0SIM July 2021

% close all;

%% settings ====================================================================
useAzimA=1;  % select azimuth and distance info of WSPR reporter A

%% select files to use =========================================================
% A is the location to measure
% relative to B as reference (e.g. WebSDR Maasbree)
load('ResultsA');
load('ResultsB');

fprintf("allData loaded. \n");

%% filter settings =============================================================
% filter on location
Floc='';
% Floc='JN80';

% pass/stop filter call (e.g. for removing outliers)
pass=0;
Fcall='';
%Fcall='EC5M';

% filter on band
Fband='';
% Fband='03';

% filter on max and min SNR
SNRmin='';
SNRmax='';
%SNRmin='-28';
%SNRmax='35';

% filter on date (YYYYMMDD) and time (HHMM)
startD='';
endD='';
%startD='20210304';
%endD='20210304';
startT='';
endT='';
%startT='1000';
%endT='2200';

% filter on max and min distance
startDist='20';
endDist='20000';

% filter on min number of measurements per call (for removing outliers)
minCall=1; % select only calls with frequency higher or equal

%% filter data =================================================================
call_len=wids(1,4);
% on location
[Valid_dataA,Valid_dataB]=filterLoc(Valid_dataA_start,Valid_dataB_start,wids,Floc);

% pass/stop filter call (e.g. for removing outliers)
[Valid_dataA,Valid_dataB]=filterCall(Valid_dataA,Valid_dataB,wids,Fcall,pass,call_len);

% on band
[Valid_dataA,Valid_dataB]=filterBand(Valid_dataA,Valid_dataB,wids,Fband);

% on max and min SNR
[Valid_dataA,Valid_dataB]=filterSNR(Valid_dataA,Valid_dataB,wids,SNRmin,SNRmax);

% on date (YYYYMMDD) and time (HHMM)
[Valid_dataA,Valid_dataB]=filterTime(Valid_dataA,Valid_dataB,wids,startT,endT,startD,endD);

% on max and min distance
[Valid_dataA,Valid_dataB]=filterDistDB(Valid_dataA,Valid_dataB,wids,startDist,endDist,useAzimA);

% get only the callA info
[~,~,~,callA,~,~]=convtonum(Valid_dataA,wids);

% filter on min number of measurements per call (for removing outliers)
[Valid_dataA, Valid_dataB, uniqCalls, callfreq]=filterCallFreq(Valid_dataA, Valid_dataB, callA, minCall);


%% update the filtered info and numbers ========================================
[datesA,timesA,SNRA,callA,bandA,locA,FfreqA,azimAdb,distAdb]=convtonum(Valid_dataA,wids);
[datesB,timesB,SNRB,callB,bandB,locB,FfreqB,azimBdb,distBdb]=convtonum(Valid_dataB,wids);


%% plot results ================================================================

fig1 = figure(1);
axes1 = axes('Parent',fig1,'XTickLabel',uniqCalls, 'Fontsize',7,'XTick',1:length(uniqCalls));
hold(axes1,'on');
plot(1:size(uniqCalls,1),callfreq);
zoom xon;
grid on;
title('Histogram of matched call signs');
hold(axes1,'off');

% datesA, timesA and datesB, timesB should be equal
[DTA,Dall_numA] = converttotime(datesA,timesA);
ddt=cat(1,1,Dall_numA(2:end,:)-Dall_numA(1:end-1,:));
period_start=find(ddt~=0);
start_times=timesA(period_start,:);
start_dates=datesA(period_start,:);
[DTstart,DTall_num] = converttotime(start_dates,start_times);

figure(2)
lenSNR=size(SNRA,1);
stepsizeSNR=100;
plot(SNRA-SNRB,'.-k','MarkerSize',10); % consider SNR as power in dB
hold on;
plot(SNRA,'.r');
plot(SNRB,'.b');
ax = gca;
set (ax, "xtick", (1:stepsizeSNR:lenSNR));
set (ax, "xticklabel", DTA(1:stepsizeSNR:lenSNR,:));
axis([1 lenSNR -35 20]);
axis ("autoy");
legend('(A-B)', 'Other (A)', 'Maasbree (B)');
ylabel('SNR (dB)');
xlabel('time');
grid on;
title('All SNRs and difference in SNR');
hold off;


%% "averaging" in each 2 minute period =========================================
INDs=zeros(size(start_times,1),1);
INDe=zeros(size(start_times,1),1);
for n=1:size(start_times,1)
    [Is,~]=find(Dall_numA>=DTall_num(n,1),1,'first');
    [Ie,~]=find(Dall_numA<=DTall_num(n,1),1,'last');
    INDs(n,1)=Is; % start indices
    INDe(n,1)=Ie; % end indices
end;
PSNRAB=(10.^((SNRA-SNRB)./10));
period_ratio=zeros(size(INDs,1),1);
max_period_PA=zeros(size(INDs,1),1);
max_period_PB=zeros(size(INDs,1),1);
for n=1:size(INDs,1)
    period_ratio(n,1)=10*log10(median(PSNRAB(INDs(n,1):(INDe(n,1)))));
    max_period_PA(n,1)=max(SNRA(INDs(n,1):(INDe(n,1))));
    max_period_PB(n,1)=max(SNRB(INDs(n,1):(INDe(n,1))));
end

% Averaging over a longer time window: Twindow =================================
Tminutes=20;
Twindow=datenum(0,0,0,0,Tminutes); % even number  for averaging 
INDs_aver=zeros(size(start_times-Tminutes/2,1),1);
INDe_aver=zeros(size(start_times-Tminutes/2,1),1);
for n=1:size(start_times-Tminutes/2,1)
    [Is,~]=find(Dall_numA>=DTall_num(n,1),1,'first');
    [Ie,~]=find(Dall_numA<=(DTall_num(n,1)+(Twindow)),1,'last');
    INDs_aver(n,1)=Is; % start indices
    INDe_aver(n,1)=Ie; % end indices
end
PSNRAB_aver=(10.^((SNRA-SNRB)./10));
SNR_diff=SNRA-SNRB;
period_ratio_aver=zeros(size(INDs_aver,1),1);
period_ratio_median=zeros(size(INDs_aver,1),1);
for n=1:size(INDs_aver,1)
    period_ratio_median(n,1)=median(SNR_diff(INDs_aver(n,1):(INDe_aver(n,1))));
end

figure(3)
plot(period_ratio,'b');
hold on;
plot(period_ratio_median,'r');
lenPAB=size(INDs_aver,1);
stepsizePAB=20;
ax = gca;
set (ax, "xtick", (1:stepsizePAB:lenPAB));
set (ax, "xticklabel", DTstart(1:stepsizePAB:lenPAB,:));
axis([1 lenPAB -35 20]);
axis ("autoy");
ylabel('SNR (dB)');
xlabel('time');
grid on;
legend('2 min median', '20 min median');
title('Moving median of SNR difference');
hold off;


%% median SNR of each calls for polarscatter ===================================
medianSNR_call=zeros(size(uniqCalls,1),1);
loc_call=repmat(' ',size(uniqCalls,1),4);
azimDB_callA=zeros(size(uniqCalls,1),1);
azimDB_callB=zeros(size(uniqCalls,1),1);
distDB_callA=zeros(size(uniqCalls,1),1);
distDB_callB=zeros(size(uniqCalls,1),1);
for n=1:size(uniqCalls,1)
    SelCall=uniqCalls(n,:);
    [UseAn,~]=find((sum(callA(:,:)==SelCall,2))==call_len);
    [UseBn,~]=find((sum(callB(:,:)==SelCall,2))==call_len); % should be equal
    if ~isempty(UseAn)
    medianSNR_call(n,1)=median(SNRA(UseAn,:)-SNRB(UseBn,:));
    loc_call(n,:)=locA(UseAn(1,1),:);
    azimDB_callA(n,:)=azimAdb(UseAn(1,1),:);
    azimDB_callB(n,:)=azimBdb(UseBn(1,1),:);
    distDB_callA(n,:)=distAdb(UseAn(1,1),:);
    distDB_callB(n,:)=distBdb(UseBn(1,1),:);
    end
end


%% select which azimuth and distance to use ====================================
if (useAzimA==1)
  azimcall=azimDB_callA;
  distcall=distDB_callA;
else
  azimcall=azimDB_callB;
  distcall=distDB_callB;
end

figure(4)
minSNR=10*floor(0.1*min(medianSNR_call));
h=polar(deg2rad(azimcall),-minSNR+medianSNR_call,"*k");
set(h,'markersize',4); 
set (gca, "fontsize", 14, "linewidth", 1, "color", [1 1 1]);
view(-90,90);
axis("ij");
title('Median SNR difference of each call (dB)');
legend(cstrcat(num2str(-minSNR), 'dB = 0dB'));


ToBucket=cat(2,medianSNR_call,azimcall);
minbucket=10*floor(0.1*min(medianSNR_call));
bucket=(minbucket-0)*ones(360,1);
bucketstep=10;    % 10 degrees buckets =========
for i=0:bucketstep:359
  bucketInd=find((azimcall>=i).*(azimcall<(i+bucketstep)));
  if (~isempty(bucketInd))
  bucket((i+1):(i+bucketstep),1)=median(ToBucket(bucketInd,1));
  end
end

figure(5)
azimbuckets=0:359;
minbucket=10*floor(0.1*min(bucket));
h=polar(deg2rad(azimbuckets),-minbucket+bucket,"k");
set(h,'markersize',4); 
set (gca, "fontsize", 14, "linewidth", 1, "color", [1 1 1]);
view(-90,90);
axis("ij");
title('SNR difference(dB)');
legend(cstrcat(num2str(-minbucket), 'dB = 0dB'));


figure(6)
%h=polar(deg2rad(azimcall),(log10(distcall+1)),'*k');
h=polar(deg2rad(azimcall),(distcall+1),'*k');
%hold on;
%h=polar(deg2rad(35+useAzimA*180),175,'*r');
%hold off;
set(h,'markersize',4); 
set (gca, "fontsize", 14, "linewidth", 1, "color", [1 1 1]);
view(-90,90);
axis("ij");
%title('Azimuth and logaritmic distance location of matched calls');
title('Azimuth and distance location of matched calls');

figure(61)
plot(distDB_callA./distDB_callB,medianSNR_call,'*k');
grid on;
ylabel('SNR difference(dB)');
xlabel('distance A / distance B');
title('Distance ratio and SNR difference A-B');

figure(62)
h=polar(deg2rad(azimDB_callA),((distDB_callA+1)),'*k');
hold on;
h=polar(deg2rad(azimDB_callB),((distDB_callB+1)),'*r');
hold off;
set(h,'markersize',4); 
set (gca, "fontsize", 14, "linewidth", 1, "color", [1 1 1]);
view(-90,90);
axis("ij");
title('Azimuth and distance location of matched calls');
legend('loc A', 'loc B');

h=medianSNR_call;
hpos=h.*(h>0);
hneg=h.*(h<0);
hnul=(0*h)+(h==0);
r=distcall;
y=r.*cos(deg2rad(azimcall));
x=r.*sin(deg2rad(azimcall));
figure(63)
quiver(x,y,0*x,hpos,'b');
hold on;
quiver(x,y,0*x,hneg,'r');
quiver(x,y,hnul,0*x,'k');
quiver(-100+0*x,-150+0*y,0.1*x,0.1*y,'g');
axis square
grid on;
title('SNR difference and azimuth and distance location A');
legend('SNR positive', 'SNR negative', 'SNR = 0', 'Location PH4RTM');
hold off;


% 3D plot with distance on z axis
h=1./(distcall/10000);
r=30+medianSNR_call;
y=r.*cos(deg2rad(azimcall));
x=r.*sin(deg2rad(azimcall));
figure(7)
plot3(x,y,h,'*','MarkerSize',4);
xlim([-30 30]);
ylim([-30 30]);
axis square
rotate3d on;
grid on;


%% histogram of SNR's ==========================================================
edges=-35:1:35;
figure(8)
hist(SNRA,edges);
title('Location A: histogram of SNR');
%ylim([0 50]);
grid on;
figure(9)
hist(SNRB,edges);
title('Location B: histogram of SNR');
%ylim([0 50]);
grid on;
figure(10)
hist(SNRA-SNRB,edges);
title('A - B: histogram of SNR differences');
%ylim([0 50]);
grid on;

%% SNRA vs. SNRB ===============================================================
figure(11)
plot(SNRB,SNRA,'.b');
hold on;
plot(-40:30,-40:30,'k');
grid on;
ylabel('SNRA');
xlabel('SNRB');
xlim([-40 30]);
ylim([-40 30]);
hold off;


%% frequency drift =============================================================
[alldatesA,alltimesA,allSNRA,~,~,~,freqA]=getchardata(AlldataA, wids);
[alldatesB,alltimesB,allSNRB,~,~,~,freqB]=getchardata(AlldataB, wids);
MatchfreqAnum = -minFreq -Roffset + convToWSPRfreq(freqA(matchIndexA,:),100,1);
MatchfreqBnum = -minFreq + convToWSPRfreq(freqB(matchIndexB,:),100,1);
figure(12)
delta_freq=MatchfreqAnum-MatchfreqBnum;
plot(delta_freq);
title('Detected frequency difference for each matched call');
ylabel('frequency (Hz)');
grid on;

[freqAnum] = convToWSPRfreq(freqA,-minFreq-Roffset,1);
figure(13)
plot(freqAnum,'.','MarkerSize',10);
title('Frequency of all on location A detected signals');
ylabel('frequency (Hz)');
xlabel('time');
ylim([0 250]);
grid on;

[freqBnum] = convToWSPRfreq(freqB,-minFreq,1);
figure(14)
plot(freqBnum,'.','MarkerSize',10);
title('Frequency of all on location B detected signals');
ylabel('frequency (Hz)');
xlabel('time (UTC)');
ylim([0 250]);
grid on;

