function [collectA, collectB] = findMatch(matchA, matchB, freqA, freqB, matchdistance, wids)
% find indices of matching signals, signals received at both location at the same time
% search only in matching time periods of 2 minutes
% PA0SIM July 2021

% frequencies for telling apart double identical calls in one 2 minute period
% matchdistance: frequency has to be within xHz to match

matchfreqA=1e6*str2double((freqA));
matchfreqB=1e6*str2double((freqB));

% first search the start en length of each 2 minute time period
alldatesA=str2double((matchA(:,1:wids(1,1))));
alldatesB=str2double((matchB(:,1:wids(1,1))));
alltimesA=str2double((matchA(:,1+wids(1,1):(wids(1,1)+wids(1,2)))));
alltimesB=str2double((matchB(:,1+wids(1,1):(wids(1,1)+wids(1,2)))));
allDTA=alldatesA*10000+alltimesA;
allDTB=alldatesB*10000+alltimesB;
ddtA=allDTA(2:end,1)-allDTA(1:end-1,1);
ddtB=allDTB(2:end,1)-allDTB(1:end-1,1);
periods_startA=cat(1,1,1+find(ddtA~=0));
periods_startB=cat(1,1,1+find(ddtB~=0));
len_perA=periods_startA(2:end,1)-periods_startA(1:end-1,1);
len_perB=periods_startB(2:end,1)-periods_startB(1:end-1,1);

sA=size(matchA,1);
sB=size(matchB,1);
numC=size(matchA,2);  % number of characters

collectA=zeros(1,sA); % indices of matched detections
collectB=zeros(1,sB);

% proces up to the last common time in the measurement
LastTimeA=allDTA(periods_startA(end,1),1);
LastTimeB=allDTB(periods_startB(end,1),1);
LastTime=min(LastTimeA,LastTimeB);
periods_startA=periods_startA(logical(allDTA(periods_startA,1)<=LastTime));
periods_startB=periods_startB(logical(allDTB(periods_startB,1)<=LastTime));

bstart=1;
% A can start later than B
while (allDTA(periods_startA(1,1),1)>allDTB(periods_startB(bstart,1),1))
    bstart=bstart+1;
end

numMatched=1; % number of matched detections

for astart=1:size(periods_startA,1)-1 % search within each periods of 2 minutes
    lenA=len_perA(astart,1);
    tomatchA=matchA(periods_startA(astart,:):(periods_startA(astart,:)+lenA-1),:);
    tomatchfreqA=matchfreqA(periods_startA(astart,:):(periods_startA(astart,:)+lenA-1),:);
    
    % find next equal start time of B
    while (allDTA(periods_startA(astart,1),1)>allDTB(periods_startB(bstart,1),1))
        bstart=bstart+1;
    end
    
    lenB=len_perB(bstart,1);
    tomatchB=matchB(periods_startB(bstart,:):(periods_startB(bstart,:)+lenB-1),:);
    tomatchfreqB=matchfreqB(periods_startB(bstart,:):(periods_startB(bstart,:)+lenB-1),:);

    for k=1:lenA
    check=0;
    m=1;
    while (m<=lenB) && (check==0)
        check=(numC==sum((tomatchA(k,:)==tomatchB(m,:))));
        if check==1
            if (abs(tomatchfreqA(k,1)-tomatchfreqB(m,1))<=matchdistance)
                collectA(1,numMatched)=periods_startA(astart,1)+k-1;
                collectB(1,numMatched)=periods_startB(bstart,1)+m-1;
                numMatched=numMatched+1;
            else
                check=0;
            end
        end
        m=m+1;
    end
    end
    
end

% remove not matching signal indices
collectA=collectA(logical(collectA~=0));
collectB=collectB(logical(collectB~=0));

end

